///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2004 OSR Open Systems Resources, Inc.
//    All Rights Reserved
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        OSR_PCI.H
//
//    ABSTRACT:
//
//		This is the general include file for the NOTHING sample driver.
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __OSR_NOTHING_H__
#define __OSR_NOTHING_H__ (1) // suppress duplicate loading of this file

extern "C" {

#include <ntddk.h>

};

//
// Handy Dandy Time Macros
//

#define ABSOLUTE(wait) (wait)

#define RELATIVE(wait) (-(wait))

#define NANOSECONDS(nanos) \
(((signed __int64)(nanos)) / 100L)

#define MICROSECONDS(micros) \
(((signed __int64)(micros)) * NANOSECONDS(1000L))

#define MILLISECONDS(milli) \
(((signed __int64)(milli)) * MICROSECONDS(1000L))

#define SECONDS(seconds) \
(((signed __int64)(seconds)) * MILLISECONDS(1000L))



#include "NOTHING_IOCTL.h"
#include "wmilib.h"
#include "nothingdata.h"

#define DRIVER_NAME "Nothing"
//
// Device Extension
//
// This is the per-device component of the device object.
//
typedef struct _NOTHING_DEV_EXT
{
	//
	// Device information
	//
	PDEVICE_OBJECT		DeviceObject;

	//
	// PDO
	//
	PDEVICE_OBJECT		Pdo;

    //
    // Underlying device object (could be a filter of the PDO)
    //
    PDEVICE_OBJECT      DeviceToSendIrpsTo;

	//
	// Other cool stuff would normally go here
	//

    UNICODE_STRING      InterfaceUnicodeString;

    // 
	// Power States
	//

	SYSTEM_POWER_STATE	SystemPowerState;
	DEVICE_POWER_STATE	DevicePowerState;

    //
    // WMI Information Registration Data
    //

    WMILIB_CONTEXT      WmiLibInfo;
	BOOLEAN				WmiRegistered;

    //
    // WMI data
    //

	Nothing_Statistics	NothingPerformanceData;
    Nothing_Generic     NothingGenericData;
    ULONG               NothingDataValue;

    //
    // Timers, DPCs and WMI Event info for signaling code.
    //

    BOOLEAN             NothingEventNoDataGuidEnabled;
    BOOLEAN             NothingEventWithDataGuidEnabled;
    ULONG               EventValueReturn;
    KDPC                NothingEventNoDataDpc;
    KTIMER              NothingEventNoDataTimer;
    KDPC                NothingEventWithDataDpc;
    KTIMER              NothingEventWithDataTimer;

    //
    // IRP storage
    //
    PIRP                SystemIrp;

} NOTHING_DEVICE_EXT, *PNOTHING_DEVICE_EXT;

//
// Function Declarations
//

//
// File Nothing.c
//
VOID     NothingUnload(PDRIVER_OBJECT DriverObject);
extern "C" NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObject,PUNICODE_STRING RegistryEntry);


//
// File NothingIO.c
//
NTSTATUS NothingDeviceControl(PDEVICE_OBJECT DeviceObject, PIRP Irp);
NTSTATUS NothingCreateClose(PDEVICE_OBJECT DeviceObject, PIRP Irp);
NTSTATUS NothingWrite(PDEVICE_OBJECT DeviceObject, PIRP Irp);
NTSTATUS NothingRead(PDEVICE_OBJECT DeviceObject, PIRP Irp);

//
// File NothingPnp.c
//
NTSTATUS NothingPnp(PDEVICE_OBJECT DeviceObject, PIRP Irp);
NTSTATUS NothingAddDevice(IN PDRIVER_OBJECT  DriverObject, IN PDEVICE_OBJECT  PhysicalDeviceObject);

//
// File NothingPower.c
//
NTSTATUS NothingPower(IN PDEVICE_OBJECT DeviceObject,IN PIRP Irp);

VOID nothingFDOPowerTransitionPoRequestComplete (IN PDEVICE_OBJECT      DeviceObject,
                                                 IN UCHAR               MinorFunction,
                                                 IN POWER_STATE         DevicePowerState,
                                                 IN PVOID               Context, 
                                                 IN PIO_STATUS_BLOCK    IoStatus);
NTSTATUS nothingFDOPowerComplete(IN PDEVICE_OBJECT  DeviceObject,
                                 IN PIRP            Irp,
                                 IN PVOID           Context);


//
// File NothingWmi.c
//
NTSTATUS NothingSystemControl(IN PDEVICE_OBJECT DeviceObject,IN PIRP Irp);
NTSTATUS NothingWmiRegistration(PNOTHING_DEVICE_EXT FdoData);
NTSTATUS NothingWmiDeRegistration(PNOTHING_DEVICE_EXT FdoData);


#endif /* __OSR_NOTHING_H__ */
