///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2004 OSR Open Systems Resources, Inc.
//    All Rights Reserved
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        NothingWMI.C -- Nothing driver V2 - WMI Functions
//
//    ABSTRACT:
//
//      This file contains the skeleton code for a WDM device driver
//      that does absolutely nothing.
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
///////////////////////////////////////////////////////////////////////////////

#include "nothing.h"
#include <initguid.h>
#include <wdmguid.h>
#include <wmistr.h>
#include <wmiguid.h>
#include <wmilib.h>
#include <wmidata.h>
#include <nothingdata.h>

#ifdef USE_BINARY_MOF_QUERY

UCHAR NothingBinaryMofData[] = {
#include "nothing.x"
};

#else

//
// Set the name of our resource in our osrscsipt.rc file.  This will
// be what WMI is looking for in our rc file.
//

#define MOFRESOURCENAME L"MofResourceName"

#endif

extern UNICODE_STRING	GlobalRegistryPath;
extern PDEVICE_OBJECT	NothingDeviceObject;


//
// THis is the index for our only WMI supported function, which is
// setting and returning our DebugLevel.
//

#define NothingPerformanceCountGuidIndex  0
#define NothingBinaryMofGuidGuidIndex     1
#define NothingGenericManagableDataIndex  2
#define NothingEventNoDataIndex           3
#define NothingEventWithDataIndex         4
#define NothingBasicMethodIndex           5

GUID NothingPerformanceGuid     = Nothing_StatisticsGuid;
GUID NothingGenericGuid         = Nothing_GenericGuid;
GUID NothingEventNoDataGuid     = Nothing_Event_NoDataGuid;
GUID NothingEventWithDataGuid   = Nothing_EventGuid;
GUID NothingBinaryMofGuid       = MSWmi_MofDataGuid;
GUID NothingBasicMethodGuid     = Nothing_InOutMethodsGuid;

//
// Return the list of the GUID's we support of WMI.  
//

WMIGUIDREGINFO NothingWmiGuidList[] =
{
    {
        &NothingPerformanceGuid, 1, 0 // performance information
    },

    {
        &NothingBinaryMofGuid,   1,
#ifdef USE_BINARY_MOF_QUERY
        0
#else
        WMIREG_FLAG_REMOVE_GUID
#endif
    },

    {
        &NothingGenericGuid, 1, 0 // Generic information
    },

    {
        &NothingEventNoDataGuid, 1, WMIREG_FLAG_EVENT_ONLY_GUID // Event information
    },

    {
        &NothingEventWithDataGuid, 1, WMIREG_FLAG_EVENT_ONLY_GUID // Event With Data information
    },

    {
        &NothingBasicMethodGuid, 1, 0 // Basic Method
    },

};

#define NothingGuidCount (sizeof(NothingWmiGuidList) / sizeof(WMIGUIDREGINFO))

//
// Forward Definitions
//

NTSTATUS NothingWmiSetDataItem(IN PDEVICE_OBJECT DeviceObject,IN PIRP Irp,
                           IN ULONG GuidIndex,IN ULONG InstanceIndex,
                           IN ULONG DataItemId,IN ULONG BufferSize,IN PUCHAR Buffer);
NTSTATUS NothingWmiQueryDataBlock(IN PDEVICE_OBJECT DeviceObject,
                                IN PIRP Irp,
                                IN ULONG GuidIndex,
                                IN ULONG InstanceIndex,
                                IN ULONG InstanceCount,
                                IN OUT PULONG InstanceLengthArray,
                                IN ULONG OutBufferSize,
                                OUT PUCHAR Buffer);
NTSTATUS NothingWmiQueryRegInfo(IN PDEVICE_OBJECT DeviceObject,
                              OUT ULONG *RegFlags,
                              OUT PUNICODE_STRING InstanceName,
                              OUT PUNICODE_STRING *RegistryPath,
                              OUT PUNICODE_STRING MofResourceName,
                              OUT PDEVICE_OBJECT *Pdo);
NTSTATUS NothingWmiSetDataBlock(IN PDEVICE_OBJECT DeviceObject,
                              IN PIRP Irp,
                              IN ULONG GuidIndex,
                              IN ULONG InstanceIndex,
                              IN ULONG BufferSize,
                              IN PUCHAR Buffer);
NTSTATUS NothingWmiExecuteMethod(IN PDEVICE_OBJECT DeviceObject,
                                 IN PIRP Irp,
                                 IN ULONG GuidIndex,
                                 IN ULONG InstanceIndex,
                                 IN ULONG MethodId,
                                 IN ULONG InBufferSize,
                                 IN ULONG OutBufferSize,
                                 IN OUT PUCHAR Buffer);
NTSTATUS NothingWmiFunctionControl(IN PDEVICE_OBJECT DeviceObject,
                                   IN PIRP Irp,
                                   IN ULONG GuidIndex,
                                   IN WMIENABLEDISABLECONTROL Function,
                                   IN BOOLEAN Enable);


VOID NothingEventNoDataDpcCallBack(PKDPC TimerDpc,PVOID Context,PVOID SA1, PVOID SA2);
VOID NothingEventWithDataDpcCallBack(PKDPC TimerDpc,PVOID Context,PVOID SA1, PVOID SA2);


///////////////////////////////////////////////////////////////////////////////
//
//  OsrWmiRegistration
//
//      Registers with WMI as a data provider for this instance of the device.
//
//  Inputs:
//
//      FdoData - Device object extension for this request.
//
//  Outputs:
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiRegistration(PNOTHING_DEVICE_EXT FdoData)
{
    NTSTATUS status;

    KdPrint(("NothingWmiRegistration Entered.\n"));

    //
    // Indicate how may GUID's we support.
    //

    FdoData->WmiLibInfo.GuidCount = sizeof (NothingWmiGuidList) /
                                 sizeof (WMIGUIDREGINFO);

    ASSERT(0 != FdoData->WmiLibInfo.GuidCount);

    //
    // Register our WMI Handlers for each request. 
    //
    
    FdoData->WmiLibInfo.GuidList = NothingWmiGuidList;
    FdoData->WmiLibInfo.QueryWmiRegInfo = NothingWmiQueryRegInfo;
    FdoData->WmiLibInfo.QueryWmiDataBlock = NothingWmiQueryDataBlock;
    FdoData->WmiLibInfo.SetWmiDataBlock = NothingWmiSetDataBlock;
    FdoData->WmiLibInfo.SetWmiDataItem = NothingWmiSetDataItem;
    FdoData->WmiLibInfo.ExecuteWmiMethod = NothingWmiExecuteMethod;
    FdoData->WmiLibInfo.WmiFunctionControl = NothingWmiFunctionControl;

    //
    // Register with WMI
    //
    
    status = IoWMIRegistrationControl(FdoData->DeviceObject,
                             WMIREG_ACTION_REGISTER
                             );


    return status;
    
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiDeRegistration
//
//      Inform WMI to remove this DeviceObject from its 
//      list of providers. This function also 
//      decrements the reference count of the deviceobject.
//
//  Inputs:
//
//      FdoData - Device object extension for this request.
//
//  Outputs:
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiDeRegistration(PNOTHING_DEVICE_EXT FdoData)
{

    KdPrint(("NothingWmiDeRegistration Entered.\n"));

    //
    // We're finished with WMI (i.e. we're going away, so unregister
    // ourselves.
    //

    return IoWMIRegistrationControl(FdoData->DeviceObject,
                                 WMIREG_ACTION_DEREGISTER
                                 );

}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingSystemControl
//
//      We have just received a System Control IRP.
//
//      Assume that this is a WMI IRP and
//      call into the WMI system library and let it handle this IRP for us.
//
//  Inputs:
//
//      DeviceObject - Device object for this request.
//
//      Irp - Irp for this request
//
//  Outputs:
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingSystemControl(IN PDEVICE_OBJECT DeviceObject,IN PIRP Irp)
{
    PNOTHING_DEVICE_EXT             fdoData;
    SYSCTL_IRP_DISPOSITION  disposition;
    NTSTATUS                status;
    PIO_STACK_LOCATION      stack;

    stack = IoGetCurrentIrpStackLocation (Irp);

    fdoData = (PNOTHING_DEVICE_EXT) DeviceObject->DeviceExtension;
    
    KdPrint(("NothingSystemControl Entered. \n"));

    //
    // Make sure that the request is for the FDO and not the PDO.
    //

    if(DeviceObject != NothingDeviceObject) {

        KdPrint(("NothingSystemControl for PDO. Exiting \n"));

        Irp->IoStatus.Status = status = STATUS_INVALID_DEVICE_REQUEST;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);
        return STATUS_INVALID_DEVICE_REQUEST;

    }

	fdoData->NothingPerformanceData.WmiIrpCount++;

    //
    // Okay, this is for the FDO. Process the request.
    //

    status = WmiSystemControl(&fdoData->WmiLibInfo, 
                                 DeviceObject, 
                                 Irp,
                                 &disposition);

    //
    // Check the disposition of the request, so that we can determine
    // what to do.
    //

    switch(disposition) {

        case IrpProcessed:
        {
            //
            // This irp has been processed and may be completed or pending.
            break;
        }
        
        case IrpNotCompleted:
        {
            //
            // This irp has not been completed, but has been fully processed.
            // we will complete it now
            IoCompleteRequest(Irp, IO_NO_INCREMENT);                
            break;
        }
        
        case IrpForward:
        case IrpNotWmi:
        {
            //
            // This irp is either not a WMI irp or is a WMI irp targetted
            // at a device lower in the stack.
            IoSkipCurrentIrpStackLocation (Irp);
            status = IoCallDriver (fdoData->DeviceToSendIrpsTo, Irp);
            break;
        }
                                    
        default:
        {
            //
            // We really should never get here, but if we do just forward....
            KdPrint(("Forwarding WMI Request.\n"));
            //OSRASSERT(FALSE);
            IoSkipCurrentIrpStackLocation (Irp);
            status = IoCallDriver (fdoData->DeviceToSendIrpsTo, Irp);
            break;
        }        
    }

    return(status);
}

//
// WMI System Call back functions
//


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiSetDataItem
//
//      This routine is a callback into the driver to set for the contents of
//      a data block. When the driver has finished filling the data block it
//      must call WmiCompleteRequest to complete the irp. The driver can
//      return STATUS_PENDING if the irp cannot be completed immediately.
//
//  Inputs:
//
//      DeviceObject is the device whose data block is being queried
//
//      Irp is the Irp that makes this request
//
//      GuidIndex is the index into the list of guids provided when the
//        device registered
//
//      InstanceIndex is the index that denotes which instance of the data block
//        is being queried.
//            
//      DataItemId has the id of the data item being set
//
//      BufferSize has the size of the data item passed
//
//      Buffer has the new values for the data item
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiSetDataItem(IN PDEVICE_OBJECT DeviceObject,
                               IN PIRP Irp,
                               IN ULONG GuidIndex,
                               IN ULONG InstanceIndex,
                               IN ULONG DataItemId,
                               IN ULONG BufferSize,
                               IN PUCHAR Buffer)
{
    PNOTHING_DEVICE_EXT fdoData;
    NTSTATUS    status;

    KdPrint(("NothingWmiSetDataItem Entered.\n"));
    
    fdoData = (PNOTHING_DEVICE_EXT) DeviceObject->DeviceExtension;

    ASSERT(InstanceIndex == 0);

    switch(GuidIndex) {

        case NothingGenericManagableDataIndex:
            {
                switch(DataItemId) {
                    case Nothing_Generic_EmbeddedData_ID:
                        {
                            if (BufferSize >= Nothing_Generic_EmbeddedData_SIZE) {
                                PNothingEmbeddedClass pEmbeddedData = (PNothingEmbeddedClass) Buffer;
                                fdoData->NothingGenericData.EmbeddedData.GenericValueMap = pEmbeddedData->GenericValueMap;
                                status = STATUS_SUCCESS;
                            } else {
                                status = STATUS_BUFFER_TOO_SMALL;
                            }
                            break;
                        }
                        break;

                    case Nothing_Generic_EmbeddedData1_ID:
                        {
                            if (BufferSize >= Nothing_Generic_EmbeddedData1_SIZE) {
                                PNothingEmbeddedClass1 pEmbeddedData1 = (PNothingEmbeddedClass1) Buffer;
                                fdoData->NothingGenericData.EmbeddedData1.Generic2 = pEmbeddedData1->Generic2;
                                status = STATUS_SUCCESS;
                            } else {
                                status = STATUS_BUFFER_TOO_SMALL;
                            }
                            break;
                        }
                        break;

                    case Nothing_Generic_EmbeddedData2_ID:
                        {
                            if (BufferSize >= Nothing_Generic_EmbeddedData2_SIZE) {
                                PNothingEmbeddedClass2 pEmbeddedData2 = (PNothingEmbeddedClass2) Buffer;
                                fdoData->NothingGenericData.EmbeddedData2.BitMask = pEmbeddedData2->BitMask;
                                status = STATUS_SUCCESS;
                            } else {
                                status = STATUS_BUFFER_TOO_SMALL;
                            }
                            break;
                        }
                        break;

                    default:
                        return STATUS_INVALID_DEVICE_REQUEST;
                        break;
                }
            }
            break;


        default:
            status = STATUS_WMI_GUID_NOT_FOUND;
            break;

    }

    status = WmiCompleteRequest(  DeviceObject,
                                  Irp,
                                  status,
                                  0,
                                  IO_NO_INCREMENT);

    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiSetDataBlock
//
//      This routine is a callback into the driver to set the contents of
//      a data block. When the driver has finished filling the data block it
//      must call WmiCompleteRequest to complete the irp. The driver can
//      return STATUS_PENDING if the irp cannot be completed immediately.
//
//  Inputs:
//
//      DeviceObject is the device whose data block is being queried
//
//      Irp is the Irp that makes this request
//
//      GuidIndex is the index into the list of guids provided when the
//        device registered
//
//      InstanceIndex is the index that denotes which instance of the data block
//        is being queried.
//            
//      DataItemId has the id of the data item being set
//
//      BufferSize has the size of the data item passed
//
//      Buffer has the new values for the data item
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiSetDataBlock(IN PDEVICE_OBJECT DeviceObject,
                              IN PIRP Irp,
                              IN ULONG GuidIndex,
                              IN ULONG InstanceIndex,
                              IN ULONG BufferSize,
                              IN PUCHAR Buffer)
{
    PNOTHING_DEVICE_EXT   fdoData;
    NTSTATUS status;

    KdPrint(("NothingWmiSetDataBlock Entered.\n"));

    fdoData = (PNOTHING_DEVICE_EXT) DeviceObject->DeviceExtension;

    ASSERT(InstanceIndex == 0);

    switch(GuidIndex) {

        case NothingGenericManagableDataIndex:
            {
                //
                // Note that this check will fail if MOF file data is not properly aligned.  Packing should
                // be 32 bit on a 32 bit machine and possibly 64 bit on a 64 bit machine...
                //
                if (BufferSize >= sizeof(Nothing_Generic)) {
                    PNothing_Generic pGenericData = (PNothing_Generic)Buffer;

                    fdoData->NothingGenericData.EmbeddedData.GenericValueMap = pGenericData->EmbeddedData.GenericValueMap;
                    fdoData->NothingGenericData.EmbeddedData1.Generic2 = pGenericData->EmbeddedData1.Generic2;
                    fdoData->NothingGenericData.EmbeddedData2.BitMask = pGenericData->EmbeddedData2.BitMask;

                    status = STATUS_SUCCESS;
                } else {
                    status = STATUS_BUFFER_TOO_SMALL;
                }
                break;
            }
            break;

        default:

            status = STATUS_WMI_GUID_NOT_FOUND;

    }

    status = WmiCompleteRequest(  DeviceObject,
                                  Irp,
                                  status,
                                  0,
                                  IO_NO_INCREMENT);

    return(status);
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiQueryDataBlock
//
//      This routine is a callback into the driver to set the contents of
//      a data block. When the driver has finished filling the data block it
//      must call WmiCompleteRequest to complete the irp. The driver can
//      return STATUS_PENDING if the irp cannot be completed immediately.
//
//  Inputs:
//
//      DeviceObject is the device whose data block is being queried
//
//      Irp is the Irp that makes this request
//
//      GuidIndex is the index into the list of guids provided when the
//        device registered
//
//      InstanceIndex is the index that denotes which instance of the data block
//        is being queried.
//            
//      DataItemId has the id of the data item being set
//
//      BufferSize has the size of the data item passed
//
//      Buffer has the new values for the data item
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiQueryDataBlock(IN PDEVICE_OBJECT DeviceObject,
                                IN PIRP Irp,
                                IN ULONG GuidIndex,
                                IN ULONG InstanceIndex,
                                IN ULONG InstanceCount,
                                IN OUT PULONG InstanceLengthArray,
                                IN ULONG OutBufferSize,
                                OUT PUCHAR Buffer)
{
    PNOTHING_DEVICE_EXT               fdoData;    
    NTSTATUS    status;
    ULONG       sizeNeeded = 0;

    KdPrint(("NothingWmiQueryDataBlock Entered.\n"));

    //
    // we only ever register 1 instance per guid
    //
    
    ASSERT((InstanceIndex == 0) && (InstanceCount == 1));
    
    fdoData = (PNOTHING_DEVICE_EXT) DeviceObject->DeviceExtension;

    switch (GuidIndex) {

        case NothingPerformanceCountGuidIndex: {
                sizeNeeded = sizeof(Nothing_Statistics);
                if (OutBufferSize >= sizeNeeded) {
                    PNothing_Statistics IrpCounter = (PNothing_Statistics)Buffer;

                    IrpCounter->TotalIrpCount = fdoData->NothingPerformanceData.TotalIrpCount;
                    IrpCounter->TotalIrpRate = fdoData->NothingPerformanceData.TotalIrpCount;
                    IrpCounter->WmiIrpCount = fdoData->NothingPerformanceData.WmiIrpCount;
                    *InstanceLengthArray = sizeNeeded;
                    status = STATUS_SUCCESS;
                } else {
                    status = STATUS_BUFFER_TOO_SMALL;
                }
                break;
            }

#ifdef USE_BINARY_MOF_QUERY
        case NothingBinaryMofGuidGuidIndex: {
                sizeNeeded = sizeof(NothingBinaryMofData);

                if (OutBufferSize < sizeNeeded)
                {
                    status = STATUS_BUFFER_TOO_SMALL;
                } else {
                    RtlCopyMemory(Buffer, NothingBinaryMofData, sizeNeeded);
                    *InstanceLengthArray = sizeNeeded;
                    status = STATUS_SUCCESS;
                }
                break;
            }
#endif

        case NothingGenericManagableDataIndex: {
                sizeNeeded = sizeof(Nothing_Generic);
                if (OutBufferSize >= sizeNeeded)
                {
                    PNothing_Generic pGenericData = (PNothing_Generic)Buffer;

                    pGenericData->EmbeddedData.GenericValueMap = fdoData->NothingGenericData.EmbeddedData.GenericValueMap;
                    pGenericData->EmbeddedData1.Generic2 = fdoData->NothingGenericData.EmbeddedData1.Generic2;
                    pGenericData->EmbeddedData2.BitMask = fdoData->NothingGenericData.EmbeddedData2.BitMask;

                    *InstanceLengthArray = sizeNeeded;
                    status = STATUS_SUCCESS;
                } else {
                    status = STATUS_BUFFER_TOO_SMALL;
                }
                break;
            }
            break;

        case NothingBasicMethodIndex: {
                //
                // Method classes do not have any data within them, but must
                // repond successfully to queries so that WMI method operation
                // work successfully.
                sizeNeeded = sizeof(USHORT);
                if (OutBufferSize < sizeNeeded)
                {
                    status = STATUS_BUFFER_TOO_SMALL;
                } else {
                    *InstanceLengthArray = sizeNeeded;
                    status = STATUS_SUCCESS;
                }
            }
            break;

        default:

            status = STATUS_WMI_GUID_NOT_FOUND;
            break;

    }

    status = WmiCompleteRequest(  DeviceObject,
                                  Irp,
                                  status,
                                  sizeNeeded,
                                  IO_NO_INCREMENT);

    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiExecuteMethod
//
//    This routine is a callback into the driver to execute a method. When the
//    driver has finished filling the data block it must call
//    WmiCompleteRequest to complete the irp. The driver can
//    return STATUS_PENDING if the irp cannot be completed immediately.
//
//  Inputs:
//
//    DeviceObject is the device whose data block is being queried
//
//    Irp is the Irp that makes this request
//
//    GuidIndex is the index into the list of guids provided when the
//        device registered
//
//    InstanceIndex is the index that denotes which instance of the data block
//        is being called.
//
//    MethodId has the id of the method being called
//
//    InBufferSize has the size of the data block passed in as the input to
//        the method.
//
//    OutBufferSize on entry has the maximum size available to write the
//        returned data block.
//
//    Buffer on entry has the input data block and on return has the output
//        output data block.
//
//
//  Outputs:
//
//    Buffer on return has the output data block.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiExecuteMethod(IN PDEVICE_OBJECT DeviceObject,
                                 IN PIRP Irp,
                                 IN ULONG GuidIndex,
                                 IN ULONG InstanceIndex,
                                 IN ULONG MethodId,
                                 IN ULONG InBufferSize,
                                 IN ULONG OutBufferSize,
                                 IN OUT PUCHAR Buffer)
{
    PNOTHING_DEVICE_EXT fdoData;
    NTSTATUS status;
    ULONG sizeNeeded = 0;
    
    KdPrint(("NothingWmiExecuteMethod Entered.\n"));

    fdoData = (PNOTHING_DEVICE_EXT)DeviceObject->DeviceExtension;

    //
    // we only ever register 1 instance per guid
    //
    
    ASSERT(InstanceIndex == 0);

    switch (GuidIndex) {

        case NothingBasicMethodIndex: {

                switch(MethodId) {

                    case GetDataValue: {
                            if(!Buffer || OutBufferSize < sizeof(ULONG)) {
                                return STATUS_INVALID_PARAMETER;
                            }
                            PULONG pLong = (PULONG) Buffer;
                            *pLong = fdoData->NothingDataValue;
                            status = STATUS_SUCCESS;
                            sizeNeeded = sizeof(ULONG);
                        }
                        break;

                    case SetDataValue: {
                            if(!Buffer || InBufferSize < sizeof(ULONG)) {
                                return STATUS_INVALID_PARAMETER;
                            }
                            PULONG pLong = (PULONG) Buffer;
                            fdoData->NothingDataValue = *pLong;
                            status = STATUS_SUCCESS;
                        }
                        break;

                    default:
                        status = STATUS_INVALID_DEVICE_REQUEST;
                        break;
                }
            }
            break;

        default:
            status = STATUS_WMI_GUID_NOT_FOUND;
            break;

    }

    status = WmiCompleteRequest(  DeviceObject,
                                  Irp,
                                  status,
                                  sizeNeeded,
                                  IO_NO_INCREMENT);

    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiFunctionControl
//
//    This routine is a callback into the driver to enabled or disable event
//    generation or data block collection. A device should only expect a
//    single enable when the first event or data consumer enables events or
//    data collection and a single disable when the last event or data
//    consumer disables events or data collection. Data blocks will only
//    receive collection enable/disable if they were registered as requiring
//    it.
//
//  Inputs:
//
//    DeviceObject is the device whose data block is being queried
//
//    GuidIndex is the index into the list of guids provided when the
//        device registered
//
//    Function specifies which functionality is being enabled or disabled
//
//    Enable is TRUE then the function is being enabled else disabled
//
//
//  Outputs:
//
//    Buffer on return has the output data block.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiFunctionControl(IN PDEVICE_OBJECT DeviceObject,
                                   IN PIRP Irp,
                                   IN ULONG GuidIndex,
                                   IN WMIENABLEDISABLECONTROL Function,
                                   IN BOOLEAN Enable)
{
    PNOTHING_DEVICE_EXT fdoData;
    NTSTATUS status;
    ULONG sizeNeeded = 0;
    
    KdPrint(("NothingWmiFunctionControl Entered.\n"));

    fdoData = (PNOTHING_DEVICE_EXT)DeviceObject->DeviceExtension;

    switch (GuidIndex) {

        case NothingPerformanceCountGuidIndex: {

            if(Function == WmiDataBlockControl) {
                    status = STATUS_SUCCESS;
//                    if(Enable) {
//                    } else {
//                    }
                } else {
                    status = STATUS_INVALID_DEVICE_REQUEST;
                }
            }
            break;

        case NothingEventNoDataIndex:  {

                if(Function == WmiEventControl) {
                    fdoData->NothingEventNoDataGuidEnabled = Enable;
                    if(Enable) {
                        LARGE_INTEGER dueTime;
                        LONG period = (LONG) SECONDS(5);
                        dueTime.QuadPart = RELATIVE(SECONDS(5));
                        KeInitializeDpc(&fdoData->NothingEventNoDataDpc,NothingEventNoDataDpcCallBack,fdoData);
                        KeInitializeTimer(&fdoData->NothingEventNoDataTimer);
                        KeSetTimerEx(&fdoData->NothingEventNoDataTimer,dueTime,period,&fdoData->NothingEventNoDataDpc);
                    } else {
                        KeCancelTimer(&fdoData->NothingEventNoDataTimer);
                        //KeFlushQueuedDpcs();
                    }
                    status = STATUS_SUCCESS;
                } else {
                    status = STATUS_INVALID_DEVICE_REQUEST;
                }
            }
            break;

        case NothingEventWithDataIndex: {

                if(Function == WmiEventControl) {
                    fdoData->NothingEventWithDataGuidEnabled = Enable;
                    if(Enable) {
                        LARGE_INTEGER dueTime;
                        LONG period = (LONG) SECONDS(5);
                        dueTime.QuadPart = RELATIVE(SECONDS(5));
                        KeInitializeDpc(&fdoData->NothingEventWithDataDpc,NothingEventWithDataDpcCallBack,fdoData);
                        KeInitializeTimer(&fdoData->NothingEventWithDataTimer);
                        KeSetTimerEx(&fdoData->NothingEventWithDataTimer,dueTime,period,&fdoData->NothingEventWithDataDpc);
                    } else {
                        KeCancelTimer(&fdoData->NothingEventWithDataTimer);
                        //KeFlushQueuedDpcs();
                    }
                    status = STATUS_SUCCESS;
                } else {
                    status = STATUS_INVALID_DEVICE_REQUEST;
                }
            }
            break;

        default:
            status = STATUS_WMI_GUID_NOT_FOUND;
            break;

    }

    status = WmiCompleteRequest(  DeviceObject,
                                  Irp,
                                  status,
                                  sizeNeeded,
                                  IO_NO_INCREMENT);

    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWmiQueryRegInfo
//
//      This routine is a callback into the driver to retrieve the list of
//      guids or data blocks that the driver wants to register with WMI. This
//      routine may not pend or block. Driver should NOT call
//      WmiCompleteRequest.
//
//  Inputs:
//
//      DeviceObject is the device whose data block is being queried
//
//      *RegFlags returns with a set of flags that describe the guids being
//          registered for this device. If the device wants enable and disable
//          collection callbacks before receiving queries for the registered
//          guids then it should return the WMIREG_FLAG_EXPENSIVE flag. Also the
//          returned flags may specify WMIREG_FLAG_INSTANCE_PDO in which case
//          the instance name is determined from the PDO associated with the
//          device object. Note that the PDO must have an associated devnode. If
//          WMIREG_FLAG_INSTANCE_PDO is not set then Name must return a unique
//          name for the device.
//
//     InstanceName returns with the instance name for the guids if
//         WMIREG_FLAG_INSTANCE_PDO is not set in the returned *RegFlags. The
//          caller will call ExFreePool with the buffer returned.
//
//      *RegistryPath returns with the registry path of the driver
//
//      *MofResourceName returns with the name of the MOF resource attached to
//          the binary file. If the driver does not have a mof resource attached
//          then this can be returned as NULL.
//
//      *Pdo returns with the device object for the PDO associated with this
//          device if the WMIREG_FLAG_INSTANCE_PDO flag is retured in 
//          *RegFlags.
//
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == PASSIVE_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWmiQueryRegInfo(IN PDEVICE_OBJECT DeviceObject,
                              OUT ULONG *RegFlags,
                              OUT PUNICODE_STRING InstanceName,
                              OUT PUNICODE_STRING *RegistryPath,
                              OUT PUNICODE_STRING MofResourceName,
                              OUT PDEVICE_OBJECT *Pdo)
{
    PNOTHING_DEVICE_EXT fdoData;
    
    KdPrint(("NothingWmiQueryRegInfo Entered.\n"));

    fdoData = (PNOTHING_DEVICE_EXT)DeviceObject->DeviceExtension;

    *RegFlags = WMIREG_FLAG_INSTANCE_PDO;
    *RegistryPath = &GlobalRegistryPath;
    *Pdo = fdoData->Pdo;
#ifndef USE_BINARY_MOF_QUERY
    RtlInitUnicodeString(MofResourceName, MOFRESOURCENAME);
#endif 

    fdoData->WmiRegistered = TRUE;
    
    return STATUS_SUCCESS;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingEventNoDataDpcCallBack
//
//      This routine is a timer callback into the driver.  It's purpose is to
//      fire a WmiEvent so that a WMI Consumer can recevie it, if listening.
//      No Data is sent with this Event.
//
//  Inputs:
//
//      TimerDPC - address of DPC Object
//
//      Context - Device Extension address.
//
//      SA1, SA2 - Not USED.
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == DISPATCH_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
VOID NothingEventNoDataDpcCallBack(PKDPC TimerDpc,PVOID Context,PVOID SA1, PVOID SA2)
{
    PNOTHING_DEVICE_EXT pFdoData = (PNOTHING_DEVICE_EXT) Context;
    NTSTATUS status;

    KdPrint(("NothingEventNoDataDpcCallBack Entered.\n"));

    status = WmiFireEvent(pFdoData->DeviceObject,&NothingEventNoDataGuid,0,0,NULL);

    KdPrint(("NothingEventNoDataDpcCallBack Exit. %x\n",status));
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingEventWithDataDpcCallBack
//
//      This routine is a timer callback into the driver.  It's purpose is to
//      fire a WmiEvent so that a WMI Consumer can recevie it, if listening.
//      Data is sent with this Event.
//
//  Inputs:
//
//      TimerDPC - address of DPC Object
//
//      Context - Device Extension address.
//
//      SA1, SA2 - Not USED.
//
//  Outputs:
//
//      None.
//
//  Returns:
//
//      STATUS_SUCCESS if successful, error otherwise.
//
//  IRQL:
//
//      IRQL == DISPATCH_LEVEL.
//
//  Notes:
//
///////////////////////////////////////////////////////////////////////////////
VOID NothingEventWithDataDpcCallBack(PKDPC TimerDpc,PVOID Context,PVOID SA1, PVOID SA2)
{
    PNOTHING_DEVICE_EXT pFdoData = (PNOTHING_DEVICE_EXT) Context;
    NTSTATUS status;
    PNothing_Event pEvent = (PNothing_Event) ExAllocatePoolWithTag(NonPagedPool,sizeof(Nothing_Event),'ptVE');

    KdPrint(("NothingEventWithDataDpcCallBack Entered.\n"));

    if(pEvent) {

        RtlZeroMemory(pEvent,sizeof(Nothing_Event));
        pEvent->EventData = pFdoData->NothingPerformanceData.WmiIrpCount;

        status = WmiFireEvent(pFdoData->DeviceObject,&NothingEventWithDataGuid,0,sizeof(Nothing_Event),pEvent);

    } else {

        status = STATUS_INSUFFICIENT_RESOURCES;

    }

    KdPrint(("NothingEventWithDataDpcCallBack Exit. %x\n",status));
}